
#include "event_reporters.h"
#include "state.h"
#include "../scanner_ui/scanner_output_queue.h"

/**
   @name Output-functions for events
   @{
*/

void output_trigger_added_event(struct scan_job_t *this,
				int port)
{
	struct flood_state_context *context = 
		this->state_context;		
	
	char num_buf[128];
	
	if(!(context->events_to_report & TRIGGER_EVENT))
		return;

	scanner_output_queue_add("INFO TRIGGER_ADDED ");
	scanner_output_queue_add(this->addr_str);
	scanner_output_queue_add(" FLOOD TCP_SYN");
	sprintf(num_buf, " %d ", port);
	scanner_output_queue_add(num_buf);		
	scanner_output_queue_add("\n");
	
}

void output_negative_response(struct scan_job_t *this,
			      struct sniffed_packet_descr_t *descr)
{
	struct flood_state_context *context = 
		this->state_context;		

	if(!(context->events_to_report & NEGATIVE_RESPONSE))
		return;
	
}

void output_timeout_info(struct scan_job_t *this)
{
	struct flood_state_context *context = 
		this->state_context;		
	
	char num_buf[128];			

	if(!(context->events_to_report & TIMEOUT_UPDATED))
		return;
		
	scanner_output_queue_add("INFO TIMEOUT ");
	scanner_output_queue_add(this->addr_str);
	scanner_output_queue_add(" FLOOD");
	
	sprintf(num_buf, " %lld", context->timing_context.cur_timeout);
	scanner_output_queue_add(num_buf);	
		
	scanner_output_queue_add("\n");
	


}

void output_cwnd_updated_event(struct scan_job_t *this)
{
	struct flood_state_context *context = 
		this->state_context;		
	
	char num_buf[128];			
		
	if(!(context->events_to_report & CWND_UPDATED))
		return;
	
	scanner_output_queue_add("INFO CWND ");
	scanner_output_queue_add(this->addr_str);
	scanner_output_queue_add(" FLOOD");
	
	sprintf(num_buf, " %lld", context->timing_context.cur_cwnd);
	scanner_output_queue_add(num_buf);	
	
	sprintf(num_buf, " %lld", context->timing_context.cur_ccthresh);
	scanner_output_queue_add(num_buf);	
	
	scanner_output_queue_add("\n");
	
}

void output_trigger_received_event(struct scan_job_t *this, int method_id, int round)
{
	
	char num_buf[128];		

	scanner_output_queue_add("INFO TRIGGER_RECEIVED ");
	scanner_output_queue_add(this->addr_str);
	scanner_output_queue_add(" FLOOD ");
	scanner_output_queue_add(trigger_finding_methods[method_id]->name);
	sprintf(num_buf, " %d", round);
	scanner_output_queue_add(num_buf);
	scanner_output_queue_add("\n");
	
}

void output_dropped_trigger_event(struct scan_job_t *this, int method_id, int round)
{
	
	char num_buf[128];
	
	scanner_output_queue_add("INFO TRIGGER_DROPPED ");
	scanner_output_queue_add(this->addr_str);
	scanner_output_queue_add(" FLOOD ");
	scanner_output_queue_add(trigger_finding_methods[method_id]->name);
	sprintf(num_buf, " %d", round);
	scanner_output_queue_add(num_buf);
	scanner_output_queue_add("\n");
	
}

/** @} */
