#ifndef _EVENT_REPORTERS_H
#define _EVENT_REPORTERS_H

#include "../scan_jobs.h"
#include "../sniffer/sniffed_packet_queue.h"
#include "../trigger_state/trigger_finding_methods.h"

/**
   @name FloodStateEvents
   @{
   
   The scanner-front-end may
   request to be informed of
   any of the events listed below
   so that it can expose this
   information to the user.  
*/


/**
   When congestion-window or slow-start-threshold
   are updated:

   "INFO CWND $TARGET_IP $STATE $NEW_CWND $NEW_CCTHRESH"
*/

#define CWND_UPDATED          (1 << 1)


/**
   When timeout/trigger-rtt-average or deviation are updated:
   
   "INFO TIMEOUT $TARGET_IP $STATE $NEW_TIMEOUT $NEW_RTT_AVERAGE $NEW_DEVIATION"   
*/

#define TIMEOUT_UPDATED       (1 << 2)


/**
   When a trigger-response is received or a trigger is dropped:
    
   When a trigger-response is received:
   
   "INFO TRIGGER_RECEIVED $TARGET_IP $STATE $TRIGGER_METHOD $TRIGGER_ROUND"
   
   When a trigger is dropped:
   
   "INFO TRIGGER_DROPPED $TARGET_IP $STATE $TRIGGER_METHOD $TRIGGER_ROUND"
*/

#define TRIGGER_EVENT         (1 << 3)

/**
   When a "negative response" such as an RST-ACK or an RST are received.  
   not yet implemented.   
*/

#define NEGATIVE_RESPONSE     (1 << 4)

/** @} */

void output_trigger_added_event(struct scan_job_t *this,
				int port);

void output_negative_response(struct scan_job_t *this,
			      struct sniffed_packet_descr_t *descr);

void output_timeout_info(struct scan_job_t *this);

void output_cwnd_updated_event(struct scan_job_t *this);

void output_trigger_received_event(struct scan_job_t *this, int method_id, int round);

void output_dropped_trigger_event(struct scan_job_t *this, int method_id, int round);

#endif
