#ifndef _FLOOD_STATE_H_
#define _FLOOD_STATE_H_

#include "packet_batch.h"
#include "port_result.h"
#include "../trigger_state/trigger_finding_methods.h"
#include "../scanner_module.h"
#include "../queue.h"

#define CWND_BLOWUP_FACTOR               10000
#define INITIAL_GLOBAL_CWND          (2 * CWND_BLOWUP_FACTOR)
#define INITIAL_GLOBAL_CCTHRESH      (80000 * CWND_BLOWUP_FACTOR)


/** \addtogroup FloodStateScanJobManager
    
    Flood-state logic - I suggest to start off by reading the
    ScanJobManagerFunction \ref flood_state_scan_job_manager
    and the \ref flood_state_context to get a general overview.
    
@{ 
*/


#define NRESCANS_OF_FILTERED_PORTS           2

#define NDEFAULT_PORTS_TO_SCAN            1024
#define NUM_OLD_BATCHES_TO_STORE           100
#define SRC_PORT  61373
#define MAX_EXTRA_TRIGGERS                  40

struct flood_state_timing_context{
	
	/** number of probes in a single batch
	 * not counting the trigger
	 */
	
	u8 cur_batch_size;
	
	s64 cur_timeout;
	/** congestion window */   
	s64 cur_cwnd;            
	
	/* copies of cwnd and ccthresh
	 * before the last drop are kept
	 * to restore them if late
	 * responses are encountered. 
	 */
	
	s64 cwnd_before_drop;
	s64 ccthresh_before_drop;

	/** cong. control threshold */	
	s64 cur_ccthresh;
  
	boolean no_responses_since_drop;

	/** keeps track of number of active packets */
	unsigned int nactive_packets;	
	
};

/**
        flood_state_context-structure
		
	Holds the context of
	a flood-state-scan-job.
	This includes the configuration
	of the scan-job such as the
	ports to scan as well as state-descriptive
	values such as the trigger which
	was used for the last packet-batch sent
	by this scan-job.
	
*/


struct flood_state_context{
	
        /******************************************
	 *        Configuration                   *
	 * (not changed once the job is executed) *
	 ******************************************/
		
	unsigned int port_array_size;
	
        /** TODO: Change this name:
	   It should be named 'port_results'
	*/

	/** array big enough so that ports_to_scan[BIGGEST_PORT]
	 * is part of the array. While this may mean, that there
	 * are some unused fields in the array, it allows us
	 * O(1) on lookups which has proven to be quite important.
	 */
	
	struct port_result **ports_to_scan;
	

	/* variables used to keep track of 
	 * scanning progress */
  

	/** note that nports_to_scan may be smaller
	 * than the port_array_size.
	 */
	unsigned int nports_to_scan;
	
	/** number of ports we have already scanned. */
	
	unsigned int nports_scanned;
	
	struct flood_state_timing_context timing_context;		
				

	/** triggers used during the scan */
	
	struct trigger_instance triggers;
	
	/* number of extra triggers added during
	   the scan.
	*/
	
	unsigned int nextra_triggers;

	/* Lists used for batch-management */
  
	struct queue_t *active_batches;
	struct queue_t *recent_batches;
	

	struct batch_creator *creator;
	
	boolean initialized;
	u8 nrescanned_filtered;
	__u16 events_to_report;
	
	unsigned int max_batch_seq_received;	
	
	void *timing_algo_state;
	struct timing_algo *timing_alg;

	
	
};

/*
  Export the state.
*/

extern struct scan_job_state_t flood_state;

/** @} */

/*  
    Export the global timing-context which may be used by timing-code of
    all scan-jobs.
  
*/
extern struct flood_state_timing_context global_timing_context;

#endif
