#include "../scanner_module.h" /* SUCCESS/FAILURE */

#include "packet_receiver.h"
#include "parse_tree.h"
#include "sniffed_packet_queue.h"

static struct packet_type my_pkt;

static rwlock_t lock = RW_LOCK_UNLOCKED;

struct sniffed_packet_queue_node sniffed_packet_queue = {
	.list = LIST_HEAD_INIT(sniffed_packet_queue.list),
	.packet_descr = NULL,
	.lock = &lock,
};

struct packet_receiver_t packet_receiver = {
	.sniffed_packet_queue = &sniffed_packet_queue,
	.parse_tree_root = NULL,	
};

/** \addtogroup PacketReceiver
    @{ */

static int packet_receiver_rcv(struct sk_buff *skb, struct net_device *dev,
			       struct packet_type *p, struct net_device *orig_dev);

int packet_receiver_init(void)
{
	
	/* Initialize parse-tree */
	packet_receiver.parse_tree_root = new_parse_tree();

	if(!packet_receiver.parse_tree_root)
		return FAILURE;

	/* register packet-type with kernel */

	/* Note: ETH_P_IP means 'ETH_P_InPut'*/
	
	my_pkt.type = htons(ETH_P_IP);
	my_pkt.dev = NULL;
	my_pkt.func = packet_receiver_rcv;

	dev_add_pack(&my_pkt);

	return SUCCESS;
}


void packet_receiver_fini(void)
{
	/* unregister packet-type. */
	dev_remove_pack(&my_pkt);
	/* It is now safe to free the parse-tree */
	if(packet_receiver.parse_tree_root)
		free_parse_tree(packet_receiver.parse_tree_root);
	
}

/**
   Callback function for Linux-network-stack

   .   
*/

static int packet_receiver_rcv(struct sk_buff *skb, struct net_device *dev,
			struct packet_type *p, struct net_device *orig_dev)
{
	void *p_descr;
	walk_parse_tree(packet_receiver.parse_tree_root, skb, &p_descr);
	
	dev_kfree_skb(skb);
	
	return 0;
}

/** @} */
