#ifndef _PACKET_RECEIVER_H
#define _PACKET_RECEIVER_H

#include <linux/netdevice.h>

/** \addtogroup PacketReceiver            
    
The packet-receiver sniffs all incoming packets and
passes them through its \ref ParseTree. If the packet
is of interest, a sniffed_packet_descr is saved
the packet-receiver's sniffed_packet_queue which
other components can then access.

To make use of the packet-receiver, first initialize
it by calling \ref packet_receiver_init, then retrieve
any fetched packets from the global queue
'packet_receiver.sniffed_packet_queue' and
deinitialize the packet-receiver by calling
packet_receiver_fini. It's that simple.

@{
*/

struct packet_receiver_t{	
	struct sniffed_packet_queue_node *sniffed_packet_queue;
	struct parse_tree_node *parse_tree_root;
};

/**
   Initialize packet-receiver
   
   Initializes the parse-tree and registers a callback
   with the kernel-network stack.
   
*/

int packet_receiver_init(void);


/**
   Deinitialize packet-receiver
   
   Removes callback from kernel-network-stack
   and deinitialize parse-tree.
*/

void packet_receiver_fini(void);

extern struct packet_receiver_t packet_receiver;

/** @} */

#endif
