#include "state.h"
#include "trigger_finding_methods.h"

#include "../scanner_module.h"
#include "../packet_submitter.h"


#include <linux/module.h>
#include <linux/list.h>


#include "icmp_er_method.h"
#include "icmp_ts_method.h"
#include "icmp_addr_mask_method.h"
#include "udp_method.h"
#include "tcp_syn_method.h"


/********************************************
  Interface to trigger-finding-method-lists.
*********************************************/

int trigger_finding_method_list_init(struct scan_job_t *this)
{
	struct trigger_state_context *context = this->state_context;

	struct trigger_instance *node[N_FINDING_METHODS];
	int t;
	
	/* By default, use all methods */
	
	node[ICMP_ER_FINDING_METHOD] =
		new_trigger_instance(ICMP_ER_FINDING_METHOD, ALL_ROUNDS);
	
	node[ICMP_TS_FINDING_METHOD] = 
		new_trigger_instance(ICMP_TS_FINDING_METHOD, ALL_ROUNDS);

	node[ICMP_ADDR_MASK_FINDING_METHOD] = 
		new_trigger_instance(ICMP_ADDR_MASK_FINDING_METHOD, ALL_ROUNDS);


	/*
	  The udp-trigger is either buggy or we're having trouble
	  with icmp-error-packet-rate-limiting. Let's take it
	  out of the default for now.
	*/

	
	
	node[UDP_FINDING_METHOD] = 
		new_trigger_instance(UDP_FINDING_METHOD, ALL_ROUNDS);
		  	
	
	node[TCP_SYN_FINDING_METHOD] = 
		new_trigger_instance(TCP_SYN_FINDING_METHOD, ALL_ROUNDS);
	
	

	for( t = 0;  t < N_FINDING_METHODS; t++)
		if(!node[t]) return FAILURE;

	list_add_tail(&node[TCP_SYN_FINDING_METHOD]->list,
		      &context->trigger_methods_list.list);
	
	list_add(&node[ICMP_ER_FINDING_METHOD]->list,
		 &context->trigger_methods_list.list);
	list_add_tail(&node[ICMP_TS_FINDING_METHOD]->list,
		      &context->trigger_methods_list.list);
	list_add_tail(&node[ICMP_ADDR_MASK_FINDING_METHOD]->list,
		      &context->trigger_methods_list.list);
	
	
	
	list_add_tail(&node[UDP_FINDING_METHOD]->list,
		      &context->trigger_methods_list.list);
	
	
	

	return SUCCESS;
	
}


struct trigger_instance *trigger_finding_method_list_head(struct scan_job_t *this)
{
	struct trigger_instance *entry;
	struct trigger_state_context *context = this->state_context;
	struct trigger_instance *cur_list = 
		& (context->trigger_methods_list);
	
	
	if(list_empty(&cur_list->list))
		return NULL;

	entry = list_entry(cur_list->list.next,
			   struct trigger_instance, list);

	
	list_del(&entry->list);
	
	return entry;
	
}

void trigger_finding_method_list_clear(struct trigger_instance *root)
{
	struct list_head *p, *n;
	
	list_for_each_safe(p, n, &root->list){
		struct trigger_instance *item
			= list_entry(p, struct trigger_instance, list);
		
		/* remove node from list... */
		list_del(p);
		/* and now delete it. */
		del_trigger_instance(item);
		

	}
	
}


struct trigger_instance *
get_trigger_node_by_method_id(struct trigger_instance *root, int method_id)
{
	struct list_head *p;
	struct trigger_instance *item;
	
	list_for_each(p, &root->list){
		item = list_entry(p, struct trigger_instance, list);
		if(item->method_id == method_id )
			return item;
	}

	return NULL;
}

int method_id_by_method_name(const char *name)
{
	int t;
	for(t = 0; t < N_FINDING_METHODS; t++)
		if(strcmp(trigger_finding_methods[t]->name, name) == 0)
			return t;

	return NO_FINDING_METHOD;
}


/*
	This array maps triggger-finding-method-ids to
	trigger-finding-method-structures.
*/

struct trigger_finding_method *trigger_finding_methods[] = {
	
	&icmp_er_method,
	&icmp_ts_method,
	&icmp_addr_mask_method,
	&udp_method,
	&tcp_syn_method,
	
};
