#ifndef _TRIGGER_FINDING_METHODS_H
#define _TRIGGER_FINDING_METHODS_H

#include <linux/time.h>

#include "../scan_jobs.h"

#include "trigger_instance.h"


#define NO_FINDING_METHOD                    -1
#define ICMP_ER_FINDING_METHOD               0
#define ICMP_TS_FINDING_METHOD               1
#define ICMP_ADDR_MASK_FINDING_METHOD        2
#define UDP_FINDING_METHOD                   3
#define TCP_SYN_FINDING_METHOD               4
#define N_FINDING_METHODS                    5

#define ALL_ROUNDS                           -1

extern struct trigger_finding_method *trigger_finding_methods[];

struct packet_batch;
struct sniffed_packet_descr_t;


typedef void (*trigger_sender_func)(struct scan_job_t *this,
				    struct trigger_instance *method_instance);
typedef int (*trigger_receiver_func)(struct scan_job_t * this,
				     struct trigger_instance *method_instance);

typedef int (*trigger_is_response_func)(struct sniffed_packet_descr_t *descr,
					struct packet_batch *batch
					);

typedef void (*trigger_register_batch_id_func)(struct trigger_instance *method_instance,
					       unsigned int batch_id);

typedef int (*trigger_context_init)(struct trigger_instance *this, int round);
typedef void (*trigger_context_fini)(struct trigger_instance *this);

typedef void * (*trigger_copy_context)
(struct trigger_instance *node);

typedef int (*trigger_extract_batch_id_func)(struct sniffed_packet_descr_t *descr);

typedef int (*trigger_get_round_by_descr_func)(struct sniffed_packet_descr_t *descr);

typedef int (*trigger_get_round_func)(void *);




struct trigger_finding_method{
	
	const char *name;
	trigger_sender_func               sender_func;
	trigger_receiver_func             receiver_func;

	trigger_is_response_func          is_response;
	trigger_register_batch_id_func    register_batch_id;
	trigger_extract_batch_id_func     extract_batch_id;

	trigger_get_round_by_descr_func   get_round_by_descr;
	trigger_get_round_func            get_round;

	trigger_context_init context_init;
	trigger_context_fini  context_fini;
	
	trigger_copy_context context_copy;
 	
};

int method_id_by_method_name(const char *name);

/* functions which operate on trigger-finding-method-list */

int trigger_finding_method_list_init(struct scan_job_t *this);
struct trigger_instance *trigger_finding_method_list_head(struct scan_job_t *this);
void trigger_finding_method_list_clear(struct trigger_instance *root);

struct trigger_instance *
get_trigger_node_by_method_id(struct trigger_instance *root, int method_id);


#endif
